import _classCallCheck from '@babel/runtime/helpers/classCallCheck';
import _createClass from '@babel/runtime/helpers/createClass';
import _defineProperty from '@babel/runtime/helpers/defineProperty';
import _slicedToArray from '@babel/runtime/helpers/slicedToArray';

var RawLinkedList = /*#__PURE__*/_createClass(function RawLinkedList() {
  _classCallCheck(this, RawLinkedList);
  _defineProperty(this, "head", undefined);
  _defineProperty(this, "tail", undefined);
  _defineProperty(this, "length", 0);
});

/**
 * Returns a specified Node from the linked list..
 * @returns Returns the a shallow copy of the Node associated with the specified index. If no Node is associated with the index, undefined is returned.
 */
function _getNode(nodes, index) {
  if (index < 0 || index >= nodes.length || index == undefined || typeof index !== 'number') return undefined;
  var count;
  var current;
  if (index <= nodes.length / 2) {
    count = 0;
    current = nodes.head;
    while (count !== index) {
      if (current) {
        current = current.next;
      }
      count++;
    }
  } else {
    count = nodes.length - 1;
    current = nodes.tail;
    while (count !== index) {
      if (current) {
        current = current.previous;
      }
      count--;
    }
  }
  return current;
}

function get(nodes, index) {
  var _getNode2;
  return (_getNode2 = _getNode(nodes, index)) === null || _getNode2 === void 0 ? void 0 : _getNode2.value;
}

/**
 * Returns the item located at the specified index.
 * @param index The zero-based index of the desired code unit. A negative index will count back from the last item.
 */
function at(nodes, index) {
  if (index === undefined) return undefined;
  var signedIndexLookUp = index < 0 && Math.abs(index) <= nodes.length ? Math.abs(index * -1 - nodes.length) : index;
  if (signedIndexLookUp >= nodes.length) return undefined;
  return get(nodes, signedIndexLookUp);
}

function pop(nodes) {
  if (!nodes.head || !nodes.tail) return undefined;
  var poppedNode = nodes.tail;
  if (nodes.length === 1) {
    nodes.head = undefined;
    nodes.tail = undefined;
  } else {
    nodes.tail = poppedNode.previous;
    if (nodes.tail) {
      nodes.tail.next = undefined;
      poppedNode.previous = undefined;
    }
  }
  nodes.length--;
  return poppedNode === null || poppedNode === void 0 ? void 0 : poppedNode.value;
}

function clear(nodes) {
  while (nodes.length > 0) {
    pop(nodes);
  }
}

function _createForOfIteratorHelper$4(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray$4(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray$4(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray$4(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$4(o, minLen); }
function _arrayLikeToArray$4(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }
function pushIterator(linkedList, iterator) {
  if (iterator) {
    var _iterator = _createForOfIteratorHelper$4(iterator),
      _step;
    try {
      for (_iterator.s(); !(_step = _iterator.n()).done;) {
        var value = _step.value;
        linkedList.push(value);
      }
    } catch (err) {
      _iterator.e(err);
    } finally {
      _iterator.f();
    }
  }
}

function _isLinkedList(item) {
  return item.constructor.name === 'LinkedList';
}

// TODO: implement function operating over the nodes instead of the LinkedList
function concat(linkedList) {
  var newLinkedList = linkedList.slice(0, linkedList.size);
  for (var _len = arguments.length, items = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    items[_key - 1] = arguments[_key];
  }
  for (var _i = 0, _items = items; _i < _items.length; _i++) {
    var item = _items[_i];
    if (_isLinkedList(item)) {
      pushIterator(newLinkedList, item.values());
    } else {
      newLinkedList.push(item);
    }
  }
  return newLinkedList;
}

function shift(nodes) {
  if (!nodes.head) return undefined;
  var shiftedNode = nodes.head;
  if (nodes.length === 1) {
    nodes.head = undefined;
    nodes.tail = undefined;
  } else {
    nodes.head = shiftedNode.next;
    if (nodes.head) {
      nodes.head.previous = undefined;
      shiftedNode.next = undefined;
    }
  }
  nodes.length--;
  return shiftedNode === null || shiftedNode === void 0 ? void 0 : shiftedNode.value;
}

function deleteValue(nodes, index) {
  if (index < 0 || index >= nodes.length) return undefined;
  if (index === nodes.length - 1) return pop(nodes);
  if (index === 0) return shift(nodes);
  var previousNode = _getNode(nodes, index - 1);
  var nextNode = _getNode(nodes, index + 1);
  var removed = previousNode.next;
  previousNode.next = removed.next;
  nextNode.previous = removed.previous;
  nodes.length--;
  return removed.value;
}

var IteratorMethodType;
(function (IteratorMethodType) {
  IteratorMethodType["values"] = "values";
  IteratorMethodType["entries"] = "entries";
  IteratorMethodType["keys"] = "keys";
})(IteratorMethodType || (IteratorMethodType = {}));

function createNextFn(nodes, iterableState, kind) {
  var rawLL = nodes;
  return function next() {
    if (!iterableState.current && iterableState.count === 0) {
      iterableState.current = rawLL.head;
    }
    var isCountLargerThanLinkedListLength = iterableState.count >= rawLL.length;
    if (iterableState.current && !isCountLargerThanLinkedListLength) {
      var value = iterableState.current.value;
      var key = iterableState.count;
      var entry = [key, value];
      iterableState.count += 1;
      iterableState.current = iterableState.current.next;
      if (kind === IteratorMethodType.entries) return {
        value: entry,
        done: false
      };
      if (kind === IteratorMethodType.keys) return {
        value: key,
        done: false
      };
      return {
        value: value,
        done: false
      };
    }
    return {
      value: undefined,
      done: true
    };
  };
}

function generateIterator(kind, nodes) {
  var _ref;
  var rawLL = nodes;
  var iterableState = {
    current: undefined,
    count: 0
  };
  var _next = createNextFn(rawLL, iterableState, kind);
  return _ref = {}, _defineProperty(_ref, Symbol.iterator, function () {
    return {
      next: function next() {
        return _next();
      }
    };
  }), _defineProperty(_ref, "next", function next() {
    return _next();
  }), _ref;
}

function entries(nodes) {
  return generateIterator(IteratorMethodType.entries, nodes);
}

function _createForOfIteratorHelper$3(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray$3(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray$3(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray$3(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$3(o, minLen); }
function _arrayLikeToArray$3(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }
function every(nodes, predicate, thisArg) {
  var _iterator = _createForOfIteratorHelper$3(entries(nodes)),
    _step;
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var _step$value = _slicedToArray(_step.value, 2),
        _index = _step$value[0],
        _value = _step$value[1];
      if (predicate.call(thisArg, _value, _index, this) === false) return false;
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
  return true;
}

function fill(linkedList, value) {
  var start = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  var end = arguments.length > 3 ? arguments[3] : undefined;
  var current = linkedList.nodes.head;
  var i = 0;
  while (current) {
    if (i >= start && i < end) {
      current.value = value;
    }
    current = current.next;
    i += 1;
  }
  return linkedList;
}

function values(nodes) {
  return generateIterator(IteratorMethodType.values, nodes);
}

function _createForOfIteratorHelper$2(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray$2(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray$2(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray$2(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$2(o, minLen); }
function _arrayLikeToArray$2(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }
function filter(nodes, updateCallback, predicate, thisArg) {
  var iteration = 0;
  var _iterator = _createForOfIteratorHelper$2(values(nodes)),
    _step;
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var _value = _step.value;
      if (predicate.call(thisArg, _value, iteration, this) === true) {
        updateCallback(_value);
      }
      iteration += 1;
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
}

function _createForOfIteratorHelper$1(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray$1(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray$1(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray$1(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray$1(o, minLen); }
function _arrayLikeToArray$1(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }

// Used to implements the find. findIndex and some functions
function findValue(nodes, cb, thisArg) {
  var iteration = 0;
  var _iterator = _createForOfIteratorHelper$1(values(nodes)),
    _step;
  try {
    for (_iterator.s(); !(_step = _iterator.n()).done;) {
      var _value = _step.value;
      if (cb.call(thisArg, _value, iteration, this) === true) return [iteration, _value];
      iteration += 1;
    }
  } catch (err) {
    _iterator.e(err);
  } finally {
    _iterator.f();
  }
  return [-1, undefined];
}

// TODO: implement function operating over the nodes instead of the LinkedList
function flat(linkedList, depth) {
  if (depth < 1) {
    return linkedList.slice(0, linkedList.size);
  }
  return linkedList.reduce(function (acc, val) {
    return acc.concat(_isLinkedList(val) ? flat(val, depth - 1) : val);
  }, linkedList.slice(0, 0));
}

// TODO: implement function operating over the nodes instead of the LinkedList
function flatMap(linkedList, newLinkedList, callback, thisArg) {
  if (linkedList.nodes.head === undefined) return newLinkedList;
  var i = 0;
  var current = linkedList.nodes.head;
  while (i < linkedList.size) {
    var cbResult = callback.call(thisArg, current.value, i, linkedList);
    if (_isLinkedList(cbResult)) {
      pushIterator(newLinkedList, cbResult.values());
    } else {
      newLinkedList.push(cbResult);
    }
    if (current.next) {
      current = current.next;
    }
    i += 1;
  }
  return newLinkedList;
}

function forEach(nodes, callbackfn, thisArg) {
  if (nodes.head === undefined) return undefined;
  var i = 0;
  var current = nodes.head;
  while (i < nodes.length) {
    callbackfn.call(thisArg, current.value, i, this);
    if (current.next) {
      current = current.next;
    }
    i += 1;
  }
  return undefined;
}

function includes(nodes, searchElement) {
  var fromIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  if (nodes.head === undefined) return false;
  var i = 0;
  var current = nodes.head;
  while (i < nodes.length) {
    if (i < fromIndex - 1) {
      i += 1;
      break;
    }
    if (current.value === searchElement) return true;
    if (current.next) {
      current = current.next;
    }
    i += 1;
  }
  return false;
}

function indexOf(nodes, searchElement) {
  var fromIndex = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;
  if (nodes.head === undefined) return -1;
  var i = fromIndex;
  var current = _getNode(nodes, fromIndex);
  while (current) {
    if (current.value === searchElement) return i;
    current = current.next;
    i += 1;
  }
  return -1;
}

var LinkedListNode = /*#__PURE__*/_createClass(function LinkedListNode(value) {
  _classCallCheck(this, LinkedListNode);
  _defineProperty(this, "next", undefined);
  _defineProperty(this, "previous", undefined);
  this.value = value;
});

function pushValue(nodes, value) {
  var newNode = new LinkedListNode(value);
  if (!nodes.head || !nodes.tail) {
    nodes.head = newNode;
    nodes.tail = newNode;
  } else {
    nodes.tail.next = newNode;
    newNode.previous = nodes.tail;
    nodes.tail = newNode;
  }
  nodes.length++;
  return newNode;
}

/**
 * Appends new elements to the end of a linked list, and returns the new length of the linked list.
 * @param items New elements to add to the linked list.
 */
function push(nodes) {
  for (var _len = arguments.length, items = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    items[_key - 1] = arguments[_key];
  }
  for (var _i = 0, _items = items; _i < _items.length; _i++) {
    var value = _items[_i];
    pushValue(nodes, value);
  }
  return nodes.length;
}

function insert(nodes, index, value) {
  if (index < 0 || index > nodes.length) return false;
  if (index === nodes.length) return !!push(nodes, value);
  var newNode = new LinkedListNode(value);
  var previousNode = _getNode(nodes, index - 1);
  var nextNode = previousNode.next;
  previousNode.next = newNode;
  newNode.next = nextNode;
  newNode.previous = previousNode;
  nextNode.previous = newNode;
  nodes.length++;
  return true;
}

function reduce(nodes, callbackfn, initialValue) {
  var _this = this;
  var isInitialValueMissing = arguments.length === 2;
  if (nodes.length === 0 && isInitialValueMissing) {
    throw new TypeError('Reduce of empty linked list with no initial value');
  }
  var currentPreviousValue = initialValue;
  forEach(nodes, function (value, index) {
    if (!currentPreviousValue && isInitialValueMissing) {
      currentPreviousValue = value;
      return;
    }
    currentPreviousValue = callbackfn(currentPreviousValue, value, index, _this);
  });
  return currentPreviousValue;
}

function join(nodes) {
  var separator = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : ',';
  return reduce(nodes, function (previousValue, currentValue, index) {
    return index ? "".concat(previousValue).concat(separator).concat(currentValue) : "".concat(currentValue);
  }, '');
}

function keys(nodes) {
  return generateIterator(IteratorMethodType.keys, nodes);
}

function lastIndexOf(nodes, searchElement, fromIndex) {
  if (nodes.tail === undefined) return -1;
  var i = nodes.length - 1;
  var current = nodes.tail;
  while (current !== undefined && i > -1) {
    if (current.value === searchElement && i <= fromIndex) return i;
    i -= 1;
    if (current.previous) {
      current = current.previous;
    }
  }
  return -1;
}

function map(nodes, updateCallback, callbackfn, thisArg) {
  if (nodes.head === undefined) return;
  var i = 0;
  var current = nodes.head;
  while (i < nodes.length) {
    updateCallback(callbackfn.call(thisArg, current.value, i, this));
    if (current.next) {
      current = current.next;
    }
    i += 1;
  }
}

function _forEachReverse(nodes, cb) {
  if (nodes.head === undefined || nodes.tail === undefined) return undefined;
  var i = nodes.length;
  var current = nodes.tail;
  while (i > 0) {
    cb(current.value, i - 1);
    if (current.previous) {
      current = current.previous;
    }
    i -= 1;
  }
  return undefined;
}

function reduceRight(nodes, callbackfn, initialValue) {
  var _this = this;
  var isInitialValueMissing = arguments.length === 2;
  if (nodes.length === 0 && isInitialValueMissing) {
    throw new TypeError('Reduce of empty linked list with no initial value');
  }
  var currentPreviousValue = initialValue;
  _forEachReverse(nodes, function (value, index) {
    if (!currentPreviousValue && isInitialValueMissing) {
      currentPreviousValue = value;
      return;
    }
    currentPreviousValue = callbackfn(currentPreviousValue, value, index, _this);
  });
  return currentPreviousValue;
}

function reverse(nodes) {
  var left = nodes.head;
  var right = nodes.tail;
  var i = 0;
  while (i < nodes.length / 2) {
    var _left, _right;
    var cachedLeft = left.value;
    left.value = right.value;
    right.value = cachedLeft;
    left = (_left = left) === null || _left === void 0 ? void 0 : _left.next;
    right = (_right = right) === null || _right === void 0 ? void 0 : _right.previous;
    i += 1;
  }
}

function set(nodes, index, value) {
  var foundNode = _getNode(nodes, index);
  if (foundNode) {
    foundNode.value = value;
    return true;
  }
  return false;
}

function slice(nodes, updateCallback, start, end) {
  var initialElement = _getNode(nodes, start);
  if (initialElement === undefined || end < start) return;
  var i = start;
  var current = initialElement;
  while (i < end) {
    updateCallback(current.value);
    if (current.next) {
      current = current.next;
    }
    i += 1;
  }
}

function insertMany(linkedList, index) {
  var current = _getNode(linkedList.nodes, index);
  for (var _len = arguments.length, items = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    items[_key - 2] = arguments[_key];
  }
  if (!current) {
    linkedList.push.apply(linkedList, items);
    return linkedList.nodes.tail;
  }
  for (var _i = 0, _items = items; _i < _items.length; _i++) {
    var _current;
    var item = _items[_i];
    var newNode = new LinkedListNode(item);
    newNode.next = current.next;
    newNode.previous = current;
    current.next = newNode;
    linkedList.nodes.length++;
    current = (_current = current) === null || _current === void 0 ? void 0 : _current.next;
  }
  return current;
}

function spliceFromStart(linkedList, deleteCount) {
  var newLinkedList = linkedList.slice(0, 0);
  var i = 1;
  while (i <= deleteCount && linkedList.size > 0) {
    var shiftedElement = linkedList.shift();
    if (shiftedElement) {
      newLinkedList.push(shiftedElement);
    }
    i++;
  }
  for (var _len = arguments.length, items = new Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    items[_key - 2] = arguments[_key];
  }
  for (var _i = 0, _items = items; _i < _items.length; _i++) {
    var item = _items[_i];
    linkedList.push(item);
  }
  return newLinkedList;
}

// TODO: implement function operating over the nodes instead of the LinkedList
function splice(linkedList, start, deleteCount) {
  var normalizedStart = Math.max(start, 0);
  for (var _len = arguments.length, items = new Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
    items[_key - 3] = arguments[_key];
  }
  if (normalizedStart === 0) return spliceFromStart.apply(void 0, [linkedList, deleteCount].concat(items));
  var returnedLinkedList = linkedList.slice(0, 0);
  var currentAfterItemsInserted = insertMany.apply(void 0, [linkedList, normalizedStart - 1].concat(items));
  for (var i = 0; i < deleteCount; i++) {
    var removedNode = currentAfterItemsInserted === null || currentAfterItemsInserted === void 0 ? void 0 : currentAfterItemsInserted.next;
    if (!removedNode) break;
    returnedLinkedList.push(removedNode.value);
    if (currentAfterItemsInserted) {
      currentAfterItemsInserted.next = removedNode === null || removedNode === void 0 ? void 0 : removedNode.next;
    }
    if (currentAfterItemsInserted !== null && currentAfterItemsInserted !== void 0 && currentAfterItemsInserted.next) {
      currentAfterItemsInserted.next.previous = currentAfterItemsInserted;
    } else {
      linkedList.nodes.tail = currentAfterItemsInserted;
    }
    linkedList.nodes.length--;
  }
  return returnedLinkedList;
}

function toString(nodes) {
  return join(nodes);
}

function unshiftValue(nodes, value) {
  var newNode = new LinkedListNode(value);
  if (!nodes.head) {
    nodes.head = newNode;
    nodes.tail = newNode;
  } else {
    nodes.head.previous = newNode;
    newNode.next = nodes.head;
    nodes.head = newNode;
  }
  nodes.length++;
  return newNode;
}
function unshift(nodes) {
  var lastIndex = (arguments.length <= 1 ? 0 : arguments.length - 1) - 1;
  for (var i = lastIndex; i >= 0; i--) {
    var item = i + 1 < 1 || arguments.length <= i + 1 ? undefined : arguments[i + 1];
    unshiftValue(nodes, item);
  }
  return nodes.length;
}

function sort(nodes, compareFn) {
  if (nodes.length <= 1) return;
  var array = [];
  while (nodes.length > 0) {
    var poppedElement = pop(nodes);
    if (poppedElement) {
      array.push(poppedElement);
    }
  }
  array.sort(compareFn);
  for (var _i = 0, _array = array; _i < _array.length; _i++) {
    var value = _array[_i];
    push(nodes, value);
  }
}

var methods = /*#__PURE__*/Object.freeze({
  __proto__: null,
  at: at,
  clear: clear,
  concat: concat,
  deleteValue: deleteValue,
  entries: entries,
  every: every,
  fill: fill,
  filter: filter,
  findValue: findValue,
  flat: flat,
  flatMap: flatMap,
  forEach: forEach,
  get: get,
  includes: includes,
  indexOf: indexOf,
  insert: insert,
  join: join,
  keys: keys,
  lastIndexOf: lastIndexOf,
  map: map,
  pop: pop,
  push: push,
  reduce: reduce,
  reduceRight: reduceRight,
  reverse: reverse,
  set: set,
  shift: shift,
  slice: slice,
  splice: splice,
  toString: toString,
  unshift: unshift,
  values: values,
  sort: sort
});

var _Symbol$iterator;
function _createForOfIteratorHelper(o, allowArrayLike) { var it = typeof Symbol !== "undefined" && o[Symbol.iterator] || o["@@iterator"]; if (!it) { if (Array.isArray(o) || (it = _unsupportedIterableToArray(o)) || allowArrayLike && o && typeof o.length === "number") { if (it) o = it; var i = 0; var F = function F() {}; return { s: F, n: function n() { if (i >= o.length) return { done: true }; return { done: false, value: o[i++] }; }, e: function e(_e) { throw _e; }, f: F }; } throw new TypeError("Invalid attempt to iterate non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method."); } var normalCompletion = true, didErr = false, err; return { s: function s() { it = it.call(o); }, n: function n() { var step = it.next(); normalCompletion = step.done; return step; }, e: function e(_e2) { didErr = true; err = _e2; }, f: function f() { try { if (!normalCompletion && it.return != null) it.return(); } finally { if (didErr) throw err; } } }; }
function _unsupportedIterableToArray(o, minLen) { if (!o) return; if (typeof o === "string") return _arrayLikeToArray(o, minLen); var n = Object.prototype.toString.call(o).slice(8, -1); if (n === "Object" && o.constructor) n = o.constructor.name; if (n === "Map" || n === "Set") return Array.from(o); if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen); }
function _arrayLikeToArray(arr, len) { if (len == null || len > arr.length) len = arr.length; for (var i = 0, arr2 = new Array(len); i < len; i++) { arr2[i] = arr[i]; } return arr2; }
_Symbol$iterator = Symbol.iterator;
var LinkedList = /*#__PURE__*/function () {
  function LinkedList(iterator) {
    _classCallCheck(this, LinkedList);
    _defineProperty(this, "nodes", new RawLinkedList());
    if (iterator) {
      var _iterator = _createForOfIteratorHelper(iterator),
        _step;
      try {
        for (_iterator.s(); !(_step = _iterator.n()).done;) {
          var _value = _step.value;
          this.push(_value);
        }
      } catch (err) {
        _iterator.e(err);
      } finally {
        _iterator.f();
      }
    }
  }
  _createClass(LinkedList, [{
    key: "size",
    get: function get() {
      return this.nodes.length;
    }
  }, {
    key: "head",
    get: function get() {
      var _this$nodes$head;
      return (_this$nodes$head = this.nodes.head) === null || _this$nodes$head === void 0 ? void 0 : _this$nodes$head.value;
    }
  }, {
    key: "tail",
    get: function get() {
      var _this$nodes$tail;
      return (_this$nodes$tail = this.nodes.tail) === null || _this$nodes$tail === void 0 ? void 0 : _this$nodes$tail.value;
    }
  }, {
    key: _Symbol$iterator,
    value: function value() {
      return values(this.nodes);
    }
  }, {
    key: "values",
    value: function values$1() {
      return values(this.nodes);
    }
  }, {
    key: "keys",
    value: function keys$1() {
      return keys(this.nodes);
    }
  }, {
    key: "entries",
    value: function entries$1() {
      return entries(this.nodes);
    }
  }, {
    key: "push",
    value: function push$1() {
      for (var _len = arguments.length, items = new Array(_len), _key = 0; _key < _len; _key++) {
        items[_key] = arguments[_key];
      }
      return push.apply(methods, [this.nodes].concat(items));
    }
  }, {
    key: "unshift",
    value: function unshift$1() {
      for (var _len2 = arguments.length, items = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        items[_key2] = arguments[_key2];
      }
      return unshift.apply(methods, [this.nodes].concat(items));
    }
  }, {
    key: "pop",
    value: function pop$1() {
      return pop(this.nodes);
    }
  }, {
    key: "shift",
    value: function shift$1() {
      return shift(this.nodes);
    }
  }, {
    key: "get",
    value: function get$1(index) {
      return get(this.nodes, index);
    }
  }, {
    key: "set",
    value: function set$1(index, value) {
      return set(this.nodes, index, value);
    }
  }, {
    key: "delete",
    value: function _delete(index) {
      return deleteValue(this.nodes, index);
    }
  }, {
    key: "insert",
    value: function insert$1(index, value) {
      return insert(this.nodes, index, value);
    }
  }, {
    key: "forEach",
    value: function forEach$1(callbackfn, thisArg) {
      forEach.bind(this)(this.nodes, callbackfn, thisArg);
    }
  }, {
    key: "clear",
    value: function clear$1() {
      return clear(this.nodes);
    }
  }, {
    key: "slice",
    value: function slice$1() {
      var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var end = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.nodes.length;
      var newLinkedList = new LinkedList();
      var addValue = function addValue(currentValue) {
        return newLinkedList.push(currentValue);
      };
      slice(this.nodes, addValue, start, end);
      return newLinkedList;
    }
  }, {
    key: "copy",
    value: function copy() {
      return this.slice();
    }
  }, {
    key: "at",
    value: function at$1(index) {
      return at(this.nodes, index);
    }
  }, {
    key: "indexOf",
    value: function indexOf$1(searchElement) {
      var fromIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      return indexOf(this.nodes, searchElement, fromIndex);
    }
  }, {
    key: "lastIndexOf",
    value: function lastIndexOf$1(searchElement) {
      var fromIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.size - 1;
      return lastIndexOf(this.nodes, searchElement, fromIndex);
    }
  }, {
    key: "includes",
    value: function includes$1(searchElement) {
      var fromIndex = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      return includes(this.nodes, searchElement, fromIndex);
    }
  }, {
    key: "find",
    value: function find(predicate, thisArg) {
      var find = findValue.bind(this);
      return find(this.nodes, predicate, thisArg)[1];
    }
  }, {
    key: "findIndex",
    value: function findIndex(predicate, thisArg) {
      var find = findValue.bind(this);
      return find(this.nodes, predicate, thisArg)[0];
    }
  }, {
    key: "some",
    value: function some(cb, thisArg) {
      return this.findIndex(cb, thisArg) > -1;
    }
  }, {
    key: "every",
    value: function every$1(predicate, thisArg) {
      var every$1 = every.bind(this);
      return every$1(this.nodes, predicate, thisArg);
    }
  }, {
    key: "map",
    value: function map$1(callbackfn, thisArg) {
      var newLinkedList = new LinkedList();
      var addValue = function addValue(currentValue) {
        return newLinkedList.push(currentValue);
      };
      var map$1 = map.bind(this);
      map$1(this.nodes, addValue, callbackfn, thisArg);
      return newLinkedList;
    }
  }, {
    key: "filter",
    value: function filter$1(predicate, thisArg) {
      var result = new LinkedList();
      var addValue = function addValue(currentValue) {
        return result.push(currentValue);
      };
      var filter$1 = filter.bind(this);
      filter$1(this.nodes, addValue, predicate, thisArg);
      return result;
    }
  }, {
    key: "sort",
    value: function sort$1(compareFn) {
      sort(this.nodes, compareFn);
      return this;
    }
  }, {
    key: "reduce",
    value: function reduce$1(callbackfn, initialValue) {
      var reduce$1 = reduce.bind(this);
      return arguments.length === 1 ? reduce$1(this.nodes, callbackfn) : reduce$1(this.nodes, callbackfn, initialValue);
    }
  }, {
    key: "reduceRight",
    value: function reduceRight$1(callbackfn, initialValue) {
      var reduceRight$1 = reduceRight.bind(this);
      return arguments.length === 1 ? reduceRight$1(this.nodes, callbackfn) : reduceRight$1(this.nodes, callbackfn, initialValue);
    }
  }, {
    key: "join",
    value: function join$1() {
      var separator = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : ',';
      return join(this.nodes, separator);
    }
  }, {
    key: "toString",
    value: function toString$1() {
      return toString(this.nodes);
    }
  }, {
    key: "reverse",
    value: function reverse$1() {
      reverse.call(this, this.nodes);
      return this;
    }
  }, {
    key: "concat",
    value: function concat$1() {
      for (var _len3 = arguments.length, items = new Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
        items[_key3] = arguments[_key3];
      }
      return concat.apply(methods, [this].concat(items));
    }
  }, {
    key: "splice",
    value: function splice$1(start) {
      var deleteCount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.size;
      for (var _len4 = arguments.length, items = new Array(_len4 > 2 ? _len4 - 2 : 0), _key4 = 2; _key4 < _len4; _key4++) {
        items[_key4 - 2] = arguments[_key4];
      }
      return splice.apply(methods, [this, start, deleteCount].concat(items));
    }
  }, {
    key: "flat",
    value: function flat$1() {
      var depth = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 1;
      return flat(this, depth);
    }
  }, {
    key: "flatMap",
    value: function flatMap$1(callback, thisArg) {
      return flatMap(this, new LinkedList(), callback, thisArg);
    }
  }, {
    key: "fill",
    value: function fill$1(value) {
      var start = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;
      var end = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : this.size;
      return fill(this, value, start, end);
    }
  }], [{
    key: "isLinkedList",
    value:
    /**
     * Check if the argument is an linked list
     * @returns true if the argument is a linked list, otherwise false
     */
    function isLinkedList(linkedList) {
      return linkedList instanceof LinkedList;
    }
  }]);
  return LinkedList;
}();

export { LinkedList, LinkedListNode, RawLinkedList };
