import type { DoublyLinkedList } from './types';
import { RawLinkedList } from '../core/raw-linked-list';
export declare class LinkedList<T = any> implements DoublyLinkedList<T> {
    nodes: RawLinkedList<T>;
    /**
     * Check if the argument is an linked list
     * @returns true if the argument is a linked list, otherwise false
     */
    static isLinkedList(linkedList: any): linkedList is LinkedList<any>;
    constructor(iterator?: Iterable<T> | null);
    get size(): number;
    get head(): any;
    get tail(): any;
    [Symbol.iterator](): Iterable<T> & Iterator<T, any, undefined>;
    values(): Iterable<T> & Iterator<T, any, undefined>;
    keys(): Iterable<number> & Iterator<number, any, undefined>;
    entries(): Iterable<import("awesome-collections-core").Entry<T>> & Iterator<import("awesome-collections-core").Entry<T>, any, undefined>;
    push(...items: T[]): number;
    unshift(...items: T[]): number;
    pop(): T | undefined;
    shift(): T | undefined;
    get(index: number): T | undefined;
    set(index: number, value: T): boolean;
    delete(index: number): T | undefined;
    insert(index: number, value: T): boolean;
    forEach(callbackfn: (value: T, index: number, linkedList: LinkedList<T>) => void, thisArg?: any): void;
    clear(): void;
    slice(start?: number, end?: number): LinkedList<T>;
    copy(): LinkedList<T>;
    at(index: number): any;
    indexOf<T = any>(searchElement: T, fromIndex?: number): number;
    lastIndexOf(searchElement: T, fromIndex?: number): number;
    includes<T>(searchElement: T, fromIndex?: number): boolean;
    find<S extends T>(predicate: (this: void, value: T, index: number, obj: LinkedList<T>) => value is S, thisArg?: any): S | undefined;
    find(predicate: (value: T, index: number, obj: LinkedList<T>) => unknown, thisArg?: any): T | undefined;
    findIndex(predicate: (value: T, index: number, obj: LinkedList<T>) => boolean, thisArg?: any): number;
    some(cb: (value: T, index: number, obj: LinkedList<T>) => boolean, thisArg?: any): boolean;
    every<S extends T>(predicate: (value: T, index: number, obj: LinkedList<T>) => value is S, thisArg?: any): this is LinkedList<S>;
    every(predicate: (value: T, index: number, obj: LinkedList<T>) => unknown, thisArg?: any): boolean;
    map<U>(callbackfn: (value: T, index?: number | undefined, linkedList?: DoublyLinkedList<T> | undefined) => U, thisArg?: any): LinkedList<U>;
    filter<S extends T>(predicate: (value: T, index: number, obj: LinkedList<T>) => value is S, thisArg?: any): LinkedList<S>;
    filter(predicate: (value: T, index: number, obj: LinkedList<T>) => unknown, thisArg?: any): LinkedList<T>;
    sort(compareFn?: ((a: T, b: T) => number) | undefined): this;
    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, LinkedList: LinkedList<T>) => T): T;
    reduce(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, LinkedList: LinkedList<T>) => T, initialValue: T): T;
    reduce<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, LinkedList: LinkedList<T>) => U, initialValue: U): U;
    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, LinkedList: LinkedList<T>) => T): T;
    reduceRight(callbackfn: (previousValue: T, currentValue: T, currentIndex: number, LinkedList: LinkedList<T>) => T, initialValue: T): T;
    reduceRight<U>(callbackfn: (previousValue: U, currentValue: T, currentIndex: number, LinkedList: LinkedList<T>) => U, initialValue: U): U;
    join(separator?: string): string;
    toString(): string;
    reverse(): this;
    concat(...items: (LinkedList<T> | T)[]): LinkedList<T>;
    splice(start: number, deleteCount?: number): LinkedList<T>;
    splice(start: number, deleteCount: number, ...items: T[]): LinkedList<T>;
    flat(depth?: number): LinkedList<T>;
    flatMap<U, This = undefined>(callback: (this: This, value: T, index?: number, linkedList?: LinkedList<T>) => U | LinkedList<U>, thisArg?: This): LinkedList<U>;
    fill(value: T, start?: number, end?: number): LinkedList<T>;
}
